package digital.laboratory.platform.entrustment.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import digital.laboratory.platform.common.core.exception.ValidateCodeException;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.entrustment.dto.ApprovedUpdateEntrustDTO;
import digital.laboratory.platform.entrustment.dto.EntrustAlterApplyApproveDTO;
import digital.laboratory.platform.entrustment.dto.EntrustAlterApplyDTO;
import digital.laboratory.platform.entrustment.query.EntrustAlterApplyQuery;
import digital.laboratory.platform.entrustment.service.EntrustAlterApplyService;
import digital.laboratory.platform.entrustment.vo.EntrustAlterApplyVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;


/**
 * 申请修改委托消息相关接口
 *
 * @author chenjiangbao
 * @describe 申请修改委托消息相关接口 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/papp/entrustAlterApply")
@Api(tags = "016-申请修改委托消息相关接口")
public class EntrustAlterApplyController {

    private final EntrustAlterApplyService entrustAlterApplyService;

    @ApiOperation(value = "查询所有申请修改委托消息分页接口", notes = "查询所有申请修改委托消息分页接口")
    @PostMapping("/page")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplyQuery')") // 委托申请修改消息查询权限
    public R<IPage<EntrustAlterApplyVO>> page(@RequestBody EntrustAlterApplyQuery query) {
        IPage<EntrustAlterApplyVO> page = entrustAlterApplyService.voPage(query);
        return R.ok(page);
    }

    @ApiOperation(value = "保存或提交委托申请修改消息", notes = "保存委托申请修改消息")
    @PostMapping("/save")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplySubmit')") // 委托申请修改消息保存提交权限
    public R save(@RequestBody @Valid EntrustAlterApplyDTO dto) {
        EntrustAlterApplyVO vo = null;
        try {
            vo = entrustAlterApplyService.save(dto);
        } catch (ValidateCodeException e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
        return R.ok(vo);
    }

    @ApiOperation(value = "修改委托申请修改消息", notes = "修改委托申请修改消息")
    @PutMapping("/update")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplyEdit')") // 委托申请修改消息修改权限
    public R update(@RequestBody @Valid EntrustAlterApplyDTO dto) {
        try {
            return entrustAlterApplyService.update(dto) ? R.ok(true,"修改成功") : R.failed(false,"修改失败");
        } catch (ValidateCodeException e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

    @ApiOperation(value = "审核提交委托申请修改消息", notes = "审核提交委托申请修改消息")
    @PutMapping("/approve")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplyApprove')") // 委托申请修改消息审核权限
    public R approve(@RequestBody @Valid EntrustAlterApplyApproveDTO approveDTO) {
        try {
            return entrustAlterApplyService.approve(approveDTO) ? R.ok(true,"操作成功") : R.failed(false,"操作成功");
        } catch (ValidateCodeException e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

    @ApiOperation(value = "删除委托申请修改消息", notes = "删除委托申请修改消息， 只能删除为提交状态的消息")
    @PostMapping("/delete")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplyDelete')") // 委托申请修改消息删除权限
    public R delete(@RequestBody List<String> ids) {
        try {
            return entrustAlterApplyService.delete(ids) ? R.ok(true, "操作成功") : R.failed(false,"操作失败");
        } catch (ValidateCodeException e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

    @ApiOperation(value = "审核通过后，调用该接口修改委托案件简要、鉴定要求,x", notes = "审核通过后，调用该接口修改委托案件简要、鉴定要求")
    @PutMapping("/updateEntrust")
    @PreAuthorize("@pms.hasPermission('EntrustAlterApplyUpdateEntrust')") // 委托申请修改消息修改委托案件简要、鉴定要求权限
    public R updateEntrust(@Valid @RequestBody ApprovedUpdateEntrustDTO dto) {
        return entrustAlterApplyService.updateEntrust(dto) ? R.ok(true, "操作成功") : R.failed(false,"操作失败");
    }

    @ApiOperation(value = "判断当前委托是否在申请中的状态", notes = "判断当前委托是否在申请中的状态")
    @GetMapping("/isApplying")
    public R isApplying(@RequestParam("entrustId") String entrustId) {
        return entrustAlterApplyService.isApplying(entrustId) ? R.ok(true, "当前委托在申请修改中！") : R.failed(false, "当前委托在未申请修改！");
    }

}
