package digital.laboratory.platform.entrustment.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import java.time.LocalDateTime;
import java.util.List;

@Data
@ApiModel(value = "ApprovedUpdateEntrustDTO", description = "审批通过后，更新委托信息的DTO类")
public class ApprovedUpdateEntrustDTO {

    @ApiModelProperty(value = "申请id")
    @NotBlank(message = "申请id不能为空!")
    private String id;

    @ApiModelProperty(value = "关联的委托id")
    @NotBlank(message = "关联的委托id不能为空!")
    private String entrustId;

    @ApiModelProperty(value = "委托日期, 鉴定委托书打印日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private LocalDateTime entrustmentTime;

    @ApiModelProperty(value = "案件名称")
    private String caseName;

    @ApiModelProperty(value = "案件简要")
    private String caseBrief;

    @ApiModelProperty(value = "鉴定要求")
    private String entrustRequirement;

    @ApiModelProperty(value = "委托检材列表")
    private List<MaterialDTO> materialList;

    @ApiModelProperty(value = "案发时间")
    private LocalDateTime happenTime;

    @ApiModelProperty(value = "委托类型，0:常规毒品，1:生物样本")
    private Integer entrustmentType;

    @ApiModelProperty(value = "是否完成， true:已完成（标记该申请已完成，该申请将不在允许修改）, false:未完成(不改变状态)")
    private Boolean finished;
}
